from datetime import date

days = [
    ['hétfő', 'hétfőn'],
    ['kedd', 'kedden'],
    ['szerda', 'szerdán'],
    ['csütörtök', 'csütörtökön'],
    ['péntek', 'pénteken'],
    ['szombat', 'szombaton'],
    ['vasárnap']
]

months = [
    'január',
    'február',
    'március',
    'április',
    'május',
    'június',
    'július',
    'augusztus',
    'szeptember',
    'október',
    'november',
    'december'
]


def is_day_of(d):
    return days[d.weekday()][-1]


def date2text(d: date, now: date):
    resp = ''
    day_diff = (d - now).days
    till_next_week = 7 - now.weekday() - 1
    till_last_week = -1 * now.weekday()

    if till_last_week - 14 < day_diff <= till_last_week - 7:
        resp += 'két hete ' + is_day_of(d)
    elif till_last_week - 7 < day_diff < till_last_week:
        resp += 'múlt héten ' + is_day_of(d)
    elif day_diff == 0:
        resp += 'ma'
    elif day_diff == 1:
        resp += 'holnap'
    elif till_last_week <= day_diff < till_next_week:
        resp += 'ezen a héten ' + is_day_of(d)
    elif till_next_week < day_diff <= till_next_week + 7:
        resp += 'jövő hét ' + is_day_of(d)
    elif till_next_week + 7 < day_diff <= till_next_week + 14:
        resp += 'két hét múlva ' + is_day_of(d)
    else:
        resp += f'{d.year}-{d.month}-{d.day}'

    return resp


def date2full_text(d: date):
    return f'{d.year} {months[d.month - 1]} {d.day}'



import pytest
from datetime import datetime

from hun_date_parser.utils import Year, Month, Day, Daypart, Hour, Minute
from hun_date_parser.date_parser.time_parsers import match_digi_clock, match_time_words, match_now, match_hwords


def test_match_digi_clock():
    fn = 'digi_clock'
    tf = [('kedden 8:45', [[Hour(8, fn), Minute(45, fn)]]),
          ('kedden 08:45', [[Hour(8, fn), Minute(45, fn)]]),
          ('ma este 18:12-kor', [[Hour(18, fn), Minute(12, fn)]])]

    for inp, exp in tf:
        out = match_digi_clock(inp)
        date_parts = []
        for e in out:
            date_parts.append(e['date_parts'])
        assert date_parts == exp


time_word_fn = 'time_words'
time_word_scenarios = [
    ('reggel nyolc előtt hat perccel', [[Hour(7, time_word_fn), Minute(54, time_word_fn)]]),
    ('reggel nyolc előtt nyolcvan perccel', [[Hour(6, time_word_fn), Minute(40, time_word_fn)]]),
    ('este 8 előtt 12 perccel', [[Hour(19, time_word_fn), Minute(48, time_word_fn)]]),
    ('nyolc óra nyolc perckor', [[Hour(8, time_word_fn), Minute(8, time_word_fn)]]),
    ('ma reggel hat óra', [[Hour(6, time_word_fn)]]),
    ('ma reggel', [[Daypart(1, time_word_fn)]]),
    ('ma délután háromkor', [[Hour(15, time_word_fn)]]),
    ('ma délután három után negyvenhat perckor', [[Hour(15, time_word_fn), Minute(46, time_word_fn)]]),
    ('ma délután haemdknc után negyvenhat perckor', [[Daypart(3, time_word_fn)]]),
    ('ma', []),
    ('ötvenöt perckor', []),
    ('húsz óra negyvenkilenc perckor', [[Hour(20, time_word_fn), Minute(49, time_word_fn)]]),
    ('20 óra negyvenkilenc perckor', [[Hour(20, time_word_fn), Minute(49, time_word_fn)]]),
    ('20 óra 49 perckor', [[Hour(20, time_word_fn), Minute(49, time_word_fn)]]),
    ('este 8-kor', [[Hour(20, time_word_fn)]]),
    ('este háromnegyed 8-kor', [[Hour(19, time_word_fn), Minute(45, time_word_fn)]]),
    ('este negyed 8-kor', [[Hour(19, time_word_fn), Minute(15, time_word_fn)]]),
    ('háromnegyed nyolckor', [[Hour(7, time_word_fn), Minute(45, time_word_fn)]]),
    ('este negyed 8 előtt 6 perccel', [[Hour(19, time_word_fn), Minute(9, time_word_fn)]]),
    ('este háromnegyed 8 előtt két perccel', [[Hour(19, time_word_fn), Minute(43, time_word_fn)]]),
    ('este fél 8 előtt harminckilenc perccel', [[Hour(18, time_word_fn), Minute(51, time_word_fn)]]),
    ('este fél 8 előtt', [[Hour(19, time_word_fn), Minute(30, time_word_fn)]]),
    ('harminckilenc perccel este fél 8 előtt', [[Hour(18, time_word_fn), Minute(51, time_word_fn)]]),
    ('mondjuk két perccel 6 után', [[Hour(18, time_word_fn), Minute(2, time_word_fn)]]),
    ('mondjuk tíz perccel 8 óra előtt', [[Hour(7, time_word_fn), Minute(50, time_word_fn)]]),
    ('délután fél négy után hat perccel', [[Hour(15, time_word_fn), Minute(36, time_word_fn)]]),
    ('6 óra után 3 perccel', [[Hour(18, time_word_fn), Minute(3, time_word_fn)]]),
    ('igen, 10-kor', [[Hour(10, time_word_fn)]]),
    ('igen 10-kor', [[Hour(10, time_word_fn)]]),
    ('ezen a héten', []),
    ('2020 december', []),
    ('kb húsz év múlva', []),
    ('kb húsz évvel ezelőtt', [])
]


@pytest.mark.parametrize("inp, exp", time_word_scenarios)
def test_match_time_words(inp, exp):
    out = match_time_words(inp)
    date_parts = []
    for e in out:
        date_parts.append(e['date_parts'])
    assert date_parts == exp


def test_match_now():
    now = datetime(2020, 12, 30, 12, 1)
    fn = 'now'
    tf = [('kedden 8:45', []),
          ('most kedden', []),
          ('legyen most',
           [[Year(now.year, fn), Month(now.month, fn), Day(now.day, fn), Hour(now.hour, fn), Minute(now.minute, fn)]])]

    for inp, exp in tf:
        out = match_now(inp, now)
        date_parts = []
        for e in out:
            date_parts.append(e['date_parts'])
        assert date_parts == exp


hword_fn = 'hwords'
hword_scenarios = [
    ('16h', [[Hour(16, hword_fn)]]),
    ('16 h', []),
    ('16h-kor', [[Hour(16, hword_fn)]]),
    ('h', []),
    ('1 hely', []),
    ('jövő csütörtökön 16h', [[Hour(16, hword_fn)]])
]


@pytest.mark.parametrize("inp, exp", hword_scenarios)
def test_match_hwords(inp, exp):
    out = match_hwords(inp)
    date_parts = []
    for e in out:
        date_parts.append(e['date_parts'])
    assert date_parts == exp



"""This module handles turning datetime instances into Hungarian text."""

from datetime import datetime
from typing import Dict

from hun_date_parser.date_textualizer.date2text import date2text, date2full_text
from hun_date_parser.date_textualizer.time2text import time2absolutetexttime, time2digi, time2relitivetexttime, \
    time2lifelike


def datetime2text(input_datetime: datetime, time_precision: int = 3, now: datetime = datetime.now()):
    """
    Returns date and time textual representation candidates for input datetime object.
    :param input_datetime: Input datetime object.
    :param time_precision: Display only hours, minutes, seconds (corresponding to 1, 2, 3)
    :param now: Current timestamp to calculate relative dates.
    :return: Dictionary with the results
    """
    datetime_textualizer = DatetimeTextualizer(now=now)
    return datetime_textualizer.generate_candidates(input_datetime=input_datetime, time_precision=time_precision)


class DatetimeTextualizer:
    """
    This class handles turning datetime instances into Hungarian text.
    """

    def __init__(self, now: datetime = datetime.now()):
        """
        :param now: Current timestamp to calculate relative dates.
        """
        self.now = now

    def generate_candidates(self, input_datetime: datetime, time_precision: int = 3) -> Dict:
        """
        Generates date and time textual representation candidates for input datetime object.
        :param input_datetime: Input datetime object.
        :param time_precision: Display only hours, minutes, seconds (corresponding to 1, 2, 3)
        :return: Dictionary with the results
        """
        dates = [date2text(input_datetime.date(), self.now.date()),
                 date2full_text(input_datetime.date())]
        times = [time2absolutetexttime(input_datetime.time(), time_precision),
                 time2digi(input_datetime.time(), time_precision),
                 time2relitivetexttime(input_datetime.time(), time_precision),
                 time2lifelike(input_datetime.time(), time_precision)]

        return {
            "dates": dates,
            "times": times
        }



import pytest
from datetime import datetime

from hun_date_parser.utils import *
from hun_date_parser.date_parser.datetime_extractor import DatetimeExtractor, extend_start_end


scenarios = [
        ('ezen a héten', datetime(2020, 12, 14, 0, 0, 0), datetime(2020, 12, 20, 23, 59, 59)),
        ('legyen ma reggel nyolckor', datetime(2020, 12, 18, 8, 0, 0), datetime(2020, 12, 18, 8, 59, 59)),
        ('legyen ma', datetime(2020, 12, 18, 0, 0, 0), datetime(2020, 12, 18, 23, 59, 59)),
        ('találkozzunk szombaton reggel háromnegyed nyolckor', datetime(2020, 12, 19, 7, 45, 0),
         datetime(2020, 12, 19, 7, 45, 59)),
        ('találkozzunk szombaton háromnegyed nyolckor', datetime(2020, 12, 19, 7, 45, 0),
         datetime(2020, 12, 19, 7, 45, 59)),
        ('találkozzunk december 27-én', datetime(2020, 12, 27), datetime(2020, 12, 27, 23, 59, 59)),
        ('találkozzunk december 10-én', datetime(2020, 12, 10), datetime(2020, 12, 10, 23, 59, 59)),
        ('találkozzunk jövő héten kedden', datetime(2020, 12, 22), datetime(2020, 12, 22, 23, 59, 59)),
        ('találkozzunk jövő héten kedden reggel nyolckor', datetime(2020, 12, 22, 8), datetime(2020, 12, 22, 8, 59, 59)),
        ('ráérek jövő hét hétfőn', datetime(2020, 12, 21), datetime(2020, 12, 21, 23, 59, 59)),
        ('ráérek jövő hét hétfőn reggel 7-kor', datetime(2020, 12, 21, 7), datetime(2020, 12, 21, 7, 59, 59)),
        ('ráérek jövő hét hétfőn reggel hétkor', datetime(2020, 12, 21, 7), datetime(2020, 12, 21, 7, 59, 59)),
        ('ráérek jövő hétfőn reggel hétkor', datetime(2020, 12, 21, 7), datetime(2020, 12, 21, 7, 59, 59)),
        ('ráérek reggel hétkor', datetime(2020, 12, 18, 7), datetime(2020, 12, 18, 7, 59, 59)),
        ('hétfőn reggel hétkor', datetime(2020, 12, 14, 7), datetime(2020, 12, 14, 7, 59, 59)),
        ('reggel hétkor', datetime(2020, 12, 18, 7), datetime(2020, 12, 18, 7, 59, 59)),
        ('hétkor', datetime(2020, 12, 18, 19), datetime(2020, 12, 18, 19, 59, 59)),
        ('2021 január 5', datetime(2021, 1, 5), datetime(2021, 1, 5, 23, 59, 59)),
        ('2021 január 5 reggel 7', datetime(2021, 1, 5, 7), datetime(2021, 1, 5, 7, 59, 59)),
        ('január 5 reggel 7', datetime(2020, 1, 5, 7), datetime(2020, 1, 5, 7, 59, 59)),
        ('január 5-én', datetime(2020, 1, 5), datetime(2020, 1, 5, 23, 59, 59)),
        ('legyen most mondjuk', datetime(2020, 12, 18), datetime(2020, 12, 18, 0, 0, 59)),  # TODO: Come up with better
        ('egy óra múlva', datetime(2020, 12, 18, 1), datetime(2020, 12, 18, 1, 59, 59)),
        ('két óra múlva', datetime(2020, 12, 18, 2), datetime(2020, 12, 18, 2, 59, 59)),
        ('egy hét múlva', datetime(2020, 12, 25), datetime(2020, 12, 25, 23, 59, 59)),
        ('5 perc múlva', datetime(2020, 12, 18, 0, 5), datetime(2020, 12, 18, 0, 5, 59)),
        ('három nap múlva', datetime(2020, 12, 21), datetime(2020, 12, 21, 23, 59, 59)),
        ('csütörtök', datetime(2020, 12, 17), datetime(2020, 12, 17, 23, 59, 59)),
        ('jövő csütörtökön', datetime(2020, 12, 24), datetime(2020, 12, 24, 23, 59, 59)),
        ('jövő csütörtökön 16h', datetime(2020, 12, 24, 16), datetime(2020, 12, 24, 16, 59, 59)),
        ('igen 10-kor', datetime(2020, 12, 18, 10), datetime(2020, 12, 18, 10, 59, 59)),
    ]


@pytest.mark.parametrize("inp_txt, st, end", scenarios)
def test_datetime_extractor(inp_txt, st, end):
    now = datetime(2020, 12, 18)
    de = DatetimeExtractor(now)
    parsed_date = de.parse_datetime(inp_txt)[0]

    assert parsed_date['start_date'] == st
    assert parsed_date['end_date'] == end


def test_extend_start_end():
    inp_1 = {'start_date': [], 'end_date': [Hour(1, '')]}
    assert extend_start_end(inp_1) == inp_1

    inp_2 = {'start_date': [Month(1, ''), Hour(1, '')], 'end_date': []}
    assert extend_start_end(inp_2) == {'start_date': [Month(1, ''), Hour(1, '')],
                                       'end_date': [Month(1, ''), Hour(1, '')]}



# multi patters
R_MULTI = r'(.*)\bvagy\b(.*)|(.*)\bés\b(.*)'

# schemas
R_TOLIG = r'(.*-?t[oóöő]l\b)(.*-?ig\b)'
R_TOL = r'([:\w ]*-?t[oóöő]l\b).*'
R_IG = r'([:\w ]*-?ig\b)'

# hyper day level patterns
R_ISO_DATE = r'(\b\d{4,4})(?:[-\\/\.](1[0-2]|0?[1-9]))?(?:[-\\/\.](1[0-9]|2[0-9]|3[01]|0?[1-9]))?'
R_NAMED_MONTH = r'(j[oöő]v[oöő].*?|tavaly.*?)?(\bjan(?:\b|\.|u[aá]r){1}|\bfeb(?:\b|r\.|\.|ru[aá]r){1}|\bm[aá]r(?:\b|c\b|c\.|\.|cius){1}|\b[aá]pr(?:\b|\.|ilis){1}\bm[aá]j(?:\b|\.|us){1}|\bj[uú]n(?:\b|\.|ius){1}|\bj[uú]l(?:\b|\.|ius){1}|\baug(?:\b|\.|usztus){1}|\bszep(?:t\b|t\.|\b|\.|tember){1}|\bokt(?:\b|\.|[oó]ber){1}|\bnov(?:\b|\.|ember){1}|\bdec(?:\b|\.|ember){1})(?: ([1-3][0-9]|[1-9]))?'

R_WEEKDAY = r'(?:(el[oő]z[oő]|m[uú]lt|ezen|j[oöő]v[oöő]).*)?(h[eé]tf[oő]|kedd|szerd[aá]|cs[uü]t[oö]rt[oö]k|p[eé]ntek|szombat|vas[aá]rnap)'
R_WEEK = r'(el[oő]z[oő] h[eé]t|m[uú]lt h[eé]t|m[uú]lth[eé]t|ezen a? h[eé]t|j[oöő]v[oöő]h[eé]t|j[oöő]v[oöő] h[eé]t)'

R_TODAY = r'\b(má(?:tól|ra))\b|\b(ma)\b|\b(mai nap)\b'
R_TOMORROW = r'\b(holnap)(?!ut[aá]n)'
R_NTOMORROW = r'\b(holnaput[aá]n)'
R_YESTERDAY = r'\b(tegnap)'
R_NYESTERDAY = r'\b(tegnapel[oő]tt)'

R_NMINS_FROM_NOW = r'(([\w]*) perc m[úu]lva)'
R_NHOURS_FROM_NOW = r'(([\w]*) [oó]ra m[úu]lva)'
R_NDAYS_FROM_NOW = r'(([\w]*) nap m[úu]lva)'
R_NWEEKS_FROM_NOW = r'(([\w]*) h[eé]t m[úu]lva)'

R_YEAR = r'(tavalyel[oöő]tt|id[eé]n|j[oöő]v[oöő]re|tavaly|.*[eé]v m[uú]lva|.*[eé]vvel ezel[oő]tt|.*[eé]vvel kor[aá]bban)'

# hour level
R_AT = r'([:\w ]*-?kor)'

# hour+minute parsers
R_DIGI = r'\b([0-2]?[0-9]):([0-9][0-9])'
R_HWORDS = r'([0-2][0-9])(?: ?óra|-?kor| ?h) ?(?:([0-9][0-9])(?:(?: ?perc| ?p)[\w]*))? ?(?:([0-9][0-9])(?:(?: ?m[áa]sodperc| ?mp)[\w]*))?'
R_HWORDS_ = r'([1-2]?[0-9])?h'
R_HOUR_MIN = r'(?:(.*hajnal[i]?|.*reggel|.*d[eé]lel[oőö]tt|.*d[eé]lut[aá]n|.*este|.*[eé]jjel))? ?(?:(.*negyed|.*f[eé]l|.*h[aá]romnegyed))? ?(?:(?:\b([0-9]{1,2}|nulla|egy|kett[oöő]|h[aá]rom|n[eé]gy|[öo]t|hat|h[eé]t|nyolc|kilenc|t[ií]z|tizenegy|tizenkett[oő]|tizenh[aá]rom|tizenn[eé]gy|tizen[oö]t|tizenhat|tizenh[eé]t|tizennyolc|tizenkilenc|h[uú]sz|huszonegy|huszonkett[oöő]|huszonh[aá]rom)(?! [eé]v|perc)-?(?:kor|ra|\b)(?: [oó]ra)?)?((?:el[oő]tt|ut[aá]n)?.*perc)?)?'
R_HOUR_MIN_REV = r'(?:(.*)(?:perc.{0,4}))?? (?:(hajnal[i]?|reggel|d[eé]lel[oőö]tt|d[eé]lut[aá]n|este|[eé]jjel))? ?(negyed|f[eé]l|h[aá]romnegyed)? ?(?:([0-9]{1,2}|nulla|egy|kett[oöő]|h[aá]rom|n[eé]gy|[öo]t|hat|h[eé]t|nyolc|kilenc|t[ií]z|tizenegy|tizenkett[oő]|tizenh[aá]rom|tizenn[eé]gy|tizen[oö]t|tizenhat|tizenh[eé]t|tizennyolc|tizenkilenc|h[uú]sz|huszonegy|huszonkett[oöő]|huszonh[aá]rom)(?! [eé]v|perc)-?(?:kor|ra|\b)(?: [oó]ra)?)? ?(ut[aá]n|el[oöő]tt)?'

# R_MIN = r'(.*)(?:perc)'
# R_SEC = r'(.*)(?: ?m[áa]sodperc| ?mp)'



from typing import NamedTuple
from copy import copy
from datetime import date, timedelta

Year = NamedTuple('Year', [('x', int), ('rule', str)])
Month = NamedTuple('Month', [('x', int), ('rule', str)])
Week = NamedTuple('Week', [('x', int), ('rule', str)])
Day = NamedTuple('Day', [('x', int), ('rule', str)])
Daypart = NamedTuple('Daypart', [('x', int), ('rule', str)])
Hour = NamedTuple('Hour', [('x', int), ('rule', str)])
Minute = NamedTuple('Minute', [('x', int), ('rule', str)])
Second = NamedTuple('Second', [('x', int), ('rule', str)])


def remove_accent(s: str):
    mapping = {'á': 'a',
               'é': 'e',
               'í': 'i',
               'ó': 'o',
               'ú': 'u',
               'ö': 'o',
               'ü': 'u',
               'ő': 'o',
               'ű': 'u'}

    for a, b in mapping.items():
        s = s.replace(a, b)

    return s


def word_to_num(s: str):

    for w in s.split():
        if w.isdigit():
            return int(w)

    _s = '<DEL>' + remove_accent(copy(s))
    res = {'dec': -1, 'num': -1}
    missing = 0

    decs = [('tizen', 'tiz'),
            ('huszon', 'husz'),
            'harminc',
            'negyven',
            'otven',
            'hatvan',
            'hetven',
            'nyolcvan',
            'kilencven']

    nums = ['nulla', 'egy', ('ketto', 'ket'), 'harom', 'negy', 'ot', 'hat', 'het', 'nyolc', 'kilenc']

    for i, dec in enumerate(decs):
        if type(dec) == tuple:
            for syn in dec:
                if syn in _s:
                    res['dec'] = (i+1) * 10
                    _s = _s.replace(syn, '<DEL>')
                    break
        else:
            if dec in _s:
                res['dec'] = (i + 1) * 10
                _s = _s.replace(dec, '<DEL>')
                break

    if res['dec'] == -1:
        missing += 1
        res['dec'] = 0

    for i, num in enumerate(nums):
        if type(num) == tuple:
            for num_syn in num:
                if '<DEL>' + num_syn in _s or ' ' + num_syn in _s:
                    res['num'] = i
        else:
            if '<DEL>' + str(num) in _s or ' ' + str(num) in _s:
                res['num'] = i

    if res['num'] == -1:
        missing += 1
        res['num'] = 0

    if missing < 2:
        return res['dec'] + res['num']
    else:
        return -1


def num_to_word(num: int):
    assert 0 <= num < 60

    decs = ['tizen',
            'huszon',
            'harminc',
            'negyven',
            'ötven']

    nums = ['nulla', 'egy', 'kettő', 'három', 'négy', 'öt', 'hat', 'hét', 'nyolc', 'kilenc']

    if num == 0:
        return 'nulla'
    if num == 10:
        return 'tíz'
    if num == 20:
        return 'húsz'

    res = ''

    if num // 10:
        res += decs[(num // 10) - 1]

    if num % 10:
        res += nums[(num % 10)]

    return res


def monday_of_calenderweek(year, week):
    first = date(year, 1, 1)
    base = 1 if first.isocalendar()[1] == 1 else 8
    return first + timedelta(days=base - first.isocalendar()[2] + 7 * (week - 1))



from hun_date_parser.utils import word_to_num, num_to_word, remove_accent


def test_remove_accent():
    tf = [('aáéeíiő', 'aaeeiio')]

    for inp, exp in tf:
        assert remove_accent(inp) == exp


def test_word_to_num():
    tf = [('egy', 1),
          ('tizenegy', 11),
          ('húsz', 20),
          ('kilencvenhat', 96),
          ('random', -1),
          ('huszonketto', 22),
          ('kilencvenkilenc', 99),
          ('nulla', 0),
          ('tiz', 10),
          ('xyz negyvenhat', 46),
          ('ötvenöt perckor', 55),
          ('55 perckor', 55),
          ('8 húsz perckor', 8),
          ('8', 8),
          ('előtt nyolcvan perccel', 80),
          ('ma reggel nyolc óra', 8),
          ('két órakor', 2),
          ('délután két órakor', 2)]

    for inp, exp in tf:
        assert word_to_num(inp) == exp


def test_num_to_word():
    tf = [(1, 'egy'),
          (23, 'huszonhárom'),
          (50, 'ötven'),
          (55, 'ötvenöt'),
          (0, 'nulla')]

    for inp, exp in tf:
        assert num_to_word(inp) == exp



from hun_date_parser.date_parser.structure_parsers import match_interval, match_multi_match


def test_match_interval():
    w = [('keddtől egészen péntekig', {'start_date': 'keddtől', 'end_date': 'egészen péntekig'}),
         ('reggeltől estig', {'start_date': 'reggeltől', 'end_date': 'estig'}),
         ('kedden', {}),
         ('2020 január másodikától jövő év közepéig', {'start_date': '2020 január másodikától', 'end_date': 'jövő év közepéig'}),
         ('2020 decemberétől', {'start_date': '2020 decemberétől', 'end_date': 'OPEN'}),
         ('ma reggeltől bármikor', {'start_date': 'ma reggeltől', 'end_date': 'OPEN'}),
         ('egészen péntekig jó lesz', {'start_date': 'OPEN', 'end_date': 'egészen péntekig'}),
         ('2020-10-12-től 2020-11-01-ig', {'start_date': '2020-10-12-től', 'end_date': '2020-11-01-ig'})]

    for inp, out in w:
        assert match_interval(inp) == out


def test_match_multi_match():
    w = [('kedden és szerdán', ['kedden', 'szerdán']),
         ('2020 január vagy februárjában', ['2020 január', 'februárjában'])]

    for inp, out in w:
        assert match_multi_match(inp) == out



from hun_date_parser.date_parser.interval_restriction.restricted_parsing import extract_datetime_within_interval,\
    ExtractWithinRangeSuccess

__all__ = ['extract_datetime_within_interval', 'ExtractWithinRangeSuccess']



from datetime import datetime

from hun_date_parser.date_parser.date_parsers import Year, Month, Week, Day
from hun_date_parser.date_parser.date_parsers import (match_iso_date, match_named_month, match_relative_day, match_weekday,
                                                      match_week, match_named_year)


def test_match_iso_date():
    fn = 'match_iso_date'
    tf = [('2020-01-15', [[Year(2020, fn), Month(1, fn), Day(15, fn)]]),
          ('legyen 2020-01 elején', [[Year(2020, fn), Month(1, fn)]]),
          ('2001-ben történt', [[Year(2001, fn)]]),
          ('2020-12-30-án', [[Year(2020, fn), Month(12, fn), Day(30, fn)]]),
          ('2020-12-30-án 2020.12.29',
           [[Year(2020, fn), Month(12, fn), Day(30, fn)], [Year(2020, fn), Month(12, fn), Day(29, fn)]])]

    for inp, exp in tf:
        out = match_iso_date(inp)
        date_parts = []
        for e in out:
            date_parts.append(e['date_parts'])
        assert date_parts == exp


def test_named_month():
    fn = 'named_month'
    now = datetime(2020, 10, 1)

    tf = [('jan 20-án', [[Month(1, fn), Day(20, fn)]]),
          ('2020 febr. 4', [[Month(2, fn), Day(4, fn)]]),
          ('január', [[Month(1, fn)]]),
          (' február ', [[Month(2, fn)]]),
          ('janos', []),
          ('', []),
          ('2020 július', [[Month(7, fn)]]),
          ('2020 június - augusztus 30', [[Month(6, fn)], [Month(8, fn), Day(30, fn)]]),
          ('június 20 - július 30', [[Month(6, fn), Day(20, fn)], [Month(7, fn), Day(30, fn)]]),
          ('tavaly február ', [[Year(2019, fn), Month(2, fn)]]),
          ('legyen jövő február 12-én', [[Year(2021, fn), Month(2, fn), Day(12, fn)]]),
          ('legyen jövő év február 12-én', [[Year(2021, fn), Month(2, fn), Day(12, fn)]]),
          ('legyen jövőre február 12-én', [[Year(2021, fn), Month(2, fn), Day(12, fn)]])]

    for inp, exp in tf:
        out = match_named_month(inp, now)
        date_parts = []
        for e in out:
            date_parts.append(e['date_parts'])
        assert date_parts == exp


def test_match_relative_day():
    fn = 'relative_day'
    now = datetime(2020, 10, 1)

    tf = [('ma', [[Year(2020, fn), Month(10, fn), Day(1, fn)]]),
          ('ma-holnap', [[Year(2020, fn), Month(10, fn), Day(1, fn)], [Year(2020, fn), Month(10, fn), Day(2, fn)]]),
          ('holnapután reggel nyolc', [[Year(2020, fn), Month(10, fn), Day(3, fn)]]),
          ('legyen ma reggel', [[Year(2020, fn), Month(10, fn), Day(1, fn)]]),
          ('miért nem jöttél tegnap? na majd ma',
           [[Year(2020, fn), Month(10, fn), Day(1, fn)], [Year(2020, fn), Month(9, fn), Day(30, fn)]])]

    for inp, exp in tf:
        out = match_relative_day(inp, now)
        date_parts = []
        for e in out:
            date_parts.append(e['date_parts'])
        assert date_parts == exp


def test_match_weekday():
    fn = 'weekday'
    now = datetime(2020, 12, 11)  # friday

    tf = [('múlt vasárnap', [[Year(2020, fn), Month(12, fn), Day(6, fn)]], False),
          ('ezen a heten hetfon', [[Year(2020, fn), Month(12, fn), Day(7, fn)]], False),
          ('jövő kedden', [[Year(2020, fn), Month(12, fn), Day(15, fn)]], False),
          ('előző szombaton ', [[Year(2020, fn), Month(12, fn), Day(5, fn)]], False),
          ('miért nem jöttél tegnap? na majd ma', [], False),
          ('jövő kedden', [[Year(2020, fn), Month(12, fn), Day(15, fn)]], False),
          ('szombaton ráérek', [[Year(2020, fn), Month(12, fn), Day(12, fn)]], True),
          ('mit szólnál hétfőhöz?', [[Year(2020, fn), Month(12, fn), Day(14, fn)]], True),
          ('pénteken ráérek', [[Year(2020, fn), Month(12, fn), Day(11, fn)]], True),
          ('múlt hét kedden beszéltünk', [[Year(2020, fn), Month(12, fn), Day(1, fn)]], True),
          ('szerdán ráérek', [[Year(2020, fn), Month(12, fn), Day(16, fn)]], True),
          ('jövő héten szerdán ráérek', [[Year(2020, fn), Month(12, fn), Day(16, fn)]], True)]

    for inp, exp, expect in tf:
        out = match_weekday(inp, now, expect)
        date_parts = []
        for e in out:
            date_parts.append(e['date_parts'])
        assert date_parts == exp


def test_match_match_week():
    fn = 'week'
    now = datetime(2020, 12, 7)

    tf = [('múlthéten', [[Year(2020, fn), Week(49, fn)]]),
          ('múlt hét kedden', [[Year(2020, fn), Week(49, fn)]]),
          ('ezen a heten hetfon', [[Year(2020, fn), Week(50, fn)]]),
          ('jövőhéten', [[Year(2020, fn), Week(51, fn)]]),
          ('legyen ma', [])]

    for inp, exp in tf:
        out = match_week(inp, now)
        date_parts = []
        for e in out:
            date_parts.append(e['date_parts'])
        assert date_parts == exp


def test_match_named_year():
    fn = 'named_year'
    now = datetime(2020, 12, 7)

    tf = [('tavaly', [[Year(2019, fn)]]),
          ('múlt hét kedden', []),
          ('legyen meg még idén légyszi', [[Year(2020, fn)]]),
          ('kb két év múlva', [[Year(2022, fn)]]),
          ('tavalyelőtt történt', [[Year(2018, fn)]]),
          ('40 év múlva', [[Year(2060, fn)]]),
          ('kb három évvel ezelőtt', [[Year(2017, fn)]]),
          ('találkozzunk jövő héten szombaton', [])]

    for inp, exp in tf:
        out = match_named_year(inp, now)
        date_parts = []
        for e in out:
            date_parts.append(e['date_parts'])
        assert date_parts == exp



"""This module handles combined date and time parsing."""

from datetime import datetime, timedelta, date, time
from calendar import monthrange
from itertools import chain

from typing import Dict, List, Union, Tuple
from copy import copy

from hun_date_parser.date_parser.structure_parsers import match_multi_match, match_interval
from hun_date_parser.date_parser.date_parsers import (match_named_month, match_iso_date, match_weekday,
                                                      match_relative_day,
                                                      match_week, match_named_year, match_n_periods_compared_to_now)
from hun_date_parser.date_parser.time_parsers import match_digi_clock, match_time_words, match_now, match_hwords
from hun_date_parser.utils import Year, Month, Week, Day, Daypart, Hour, Minute, monday_of_calenderweek

datelike = Union[datetime, date, time, None]

daypart_mapping = [
    (3, 5),     # hajnal
    (6, 10),    # reggel
    (8, 11),    # délelőtt
    (12, 18),   # délután
    (18, 21),   # este
    (22, 2)     # éjjel
]


def text2datetime(input_sentence: str, now: datetime = datetime.now(),
                  expect_future_day: bool = False) -> List[Dict[str, datelike]]:
    """
    Returns the list of datetime intervals found in the input sentence.
    :param input_sentence: Input sentence string.
    :param now: Current timestamp to calculate relative dates.
    :param expect_future_day: Shows if the extracted date will be offset.
    :return: list of datetime interval dictionaries
    """
    datetime_extractor = DatetimeExtractor(now=now, output_container='datetime', expect_future_day=expect_future_day)
    return datetime_extractor.parse_datetime(sentence=input_sentence)


def text2date(input_sentence: str, now: datetime = datetime.now(),
              expect_future_day: bool = False) -> List[Dict[str, datelike]]:
    """
    Returns the list of date intervals found in the input sentence.
    :param input_sentence: Input sentence string.
    :param now: Current timestamp to calculate relative dates.
    :param expect_future_day: Shows if the extracted date will be offset.
    :return: list of date interval dictionaries
    """
    datetime_extractor = DatetimeExtractor(now=now, output_container='date', expect_future_day=expect_future_day)
    return datetime_extractor.parse_datetime(sentence=input_sentence)


def text2time(input_sentence: str, now: datetime = datetime.now(),
              expect_future_day: bool = False) -> List[Dict[str, datelike]]:
    """
    Returns the list of time intervals found in the input sentence.
    :param input_sentence: Input sentence string.
    :param now: Current timestamp to calculate relative dates.
    :param expect_future_day: Shows if the extracted date will be offset.
    :return: list of time interval dictionaries
    """
    datetime_extractor = DatetimeExtractor(now=now, output_container='time', expect_future_day=expect_future_day)
    return datetime_extractor.parse_datetime(sentence=input_sentence)


def match_rules(now: datetime, sentence: str, expect_future_day: bool = False) -> List:
    """
    Matches all rules against input text.
    :param now: Current timestamp to calculate relative dates.
    :param sentence: Input sentence.
    :param expect_future_day: Shows if the extracted date will be offset.
    :return: Parsed date and time classes.
    """
    matches = [*match_named_month(sentence, now),
               *match_iso_date(sentence),
               *match_relative_day(sentence, now),
               *match_weekday(sentence, now, expect_future_day),
               *match_week(sentence, now),
               *match_named_year(sentence, now),
               *match_digi_clock(sentence),
               *match_hwords(sentence),
               *match_time_words(sentence),
               *match_now(sentence, now),
               *match_n_periods_compared_to_now(sentence, now)]

    matches = list(chain(*[m['date_parts'] for m in matches]))

    return matches


def extend_start_end(interval: Dict) -> Dict:
    """
    Heuristic to add missing date and time classes in case of interval.
    :param interval: Dictionary with start and end dateparts.
    :return: Extended dictionary with start and end dateparts.
    """
    if interval['start_date'] == 'OPEN' or interval['end_date'] == 'OPEN':
        return interval

    interval_ = copy(interval)
    for dp in interval['start_date']:
        if type(dp) not in [type(d) for d in interval['end_date']]:
            interval_['end_date'].append(dp)

    return interval_


def type_isin_list(date_type: type, lst: Union[List, str]) -> bool:
    if type(lst) == str:
        return False

    matches = [pot_dp for pot_dp in lst if isinstance(pot_dp, date_type)]

    return bool(matches)


class DatetimeExtractor:
    """
    This class handles combined date and time parsing.
    """

    def __init__(self, now: datetime = datetime.now(), output_container: str = 'datetime',
                 expect_future_day: bool = False) -> None:
        """
        :param now: Current timestamp to calculate relative dates.
        :param output_container: datetime object to populate with datetime parts
        :param expect_future_day: Shows if the extracted date will be offset.
        """
        self.now = now
        self.output_container = output_container
        self.expect_future_day = expect_future_day

    def _get_implicit_intervall(self, sentence_part: str):
        matches = match_rules(self.now, sentence_part, self.expect_future_day)
        return [{'start_date': matches, 'end_date': matches}]

    def assamble_datetime(self, now: datetime,
                          dateparts: Union[List[Union[Year, Month, Week, Day, Daypart, Hour, Minute]], str],
                          bottom: bool = True) -> datelike:
        """
        Assambles parsed date and time classes into datetime instance.
        :param now: Current timestamp to calculate relative dates.
        :param dateparts: List of date and time classes.
        :param bottom: True if the bottom of the interval should be returned, False otherwise
        :param output_container: datetime object to populate with datetime parts
        :return: datetime instance
        """
        res_dt: List[int] = []

        if dateparts == 'OPEN':
            return None
        if not dateparts:
            return None

        has_date, has_time = False, False

        pre_first = True
        for date_type in [Year, Month, Week, Day, Daypart, Hour, Minute]:
            dp_match = [pot_dp for pot_dp in dateparts if isinstance(pot_dp, date_type)]

            _dp_match: Union[Tuple[int, str], None]
            if dp_match:
                _dp_match = dp_match[0]
            else:
                _dp_match = None

            if date_type == Year:
                if _dp_match:
                    has_date = True
                    pre_first = False
                    res_dt.append(_dp_match[0])
                else:
                    # TODO: this should take into account the expect_future_day parameter...
                    res_dt.append(now.year)

            if date_type == Month:
                if _dp_match:
                    has_date = True
                    pre_first = False
                    res_dt.append(_dp_match[0])
                elif pre_first:
                    res_dt.append(now.month)
                elif bottom:
                    res_dt.append(1)
                else:
                    res_dt.append(12)

            if date_type == Week and not type_isin_list(Day, dateparts):
                if _dp_match:
                    has_date = True
                    pre_first = False
                    week2dt = monday_of_calenderweek(res_dt[0], _dp_match[0]) + timedelta(days=(0 if bottom else 6))
                    res_dt = [week2dt.year, week2dt.month, week2dt.day]

            if date_type == Day and len(res_dt) == 2:
                if _dp_match:
                    has_date = True
                    pre_first = False
                    res_dt.append(_dp_match[0])
                elif pre_first:
                    res_dt.append(now.day)
                elif bottom:
                    res_dt.append(1)
                else:
                    mr = monthrange(res_dt[0], res_dt[1])
                    res_dt.append(mr[1])

            if date_type == Daypart:
                if _dp_match:
                    has_time = True
                    pre_first = False
                    dp = _dp_match[0]
                    if bottom:
                        res_dt.append(daypart_mapping[dp][0])
                    elif dp == 5:
                        y, m, d = res_dt
                        next_day = datetime(y, m, d) + timedelta(days=1)
                        res_dt = [next_day.year, next_day.month, next_day.day, daypart_mapping[dp][1]]
                    else:
                        res_dt.append(daypart_mapping[dp][1])

            if date_type == Hour and len(res_dt) == 3:
                if _dp_match:
                    has_time = True
                    pre_first = False
                    res_dt.append(_dp_match[0])
                elif pre_first:
                    res_dt.append(now.hour)
                elif bottom:
                    res_dt.append(0)
                elif not bottom:
                    res_dt.append(23)

            if date_type == Minute:
                if _dp_match:
                    has_time = True
                    pre_first = False
                    res_dt.append(_dp_match[0])
                elif pre_first:
                    res_dt.append(now.minute)
                elif bottom:
                    res_dt.append(0)
                elif not bottom:
                    res_dt.append(59)

        if bottom:
            res_dt.append(0)
        else:
            res_dt.append(59)

        y, m, d, h, mi, s = res_dt

        if self.output_container == 'datetime':
            if has_date or has_time:
                return datetime(y, m, d, h, mi, s)
            else:
                return None
        elif self.output_container == 'date':
            if has_date:
                return date(y, m, d)
            else:
                return None
        elif self.output_container == 'time':
            if has_time:
                return time(h, mi, s)
            else:
                return None
        else:
            return None

    def parse_datetime(self, sentence: str) -> List[Dict[str, datelike]]:
        """
        Extracts list of datetime intervals from input sentence.
        :param sentence: Input sentence string.
        :return: list of datetime interval dictionaries
        """
        sentence = sentence.lower()
        sentence_parts = match_multi_match(sentence)
        parsed_dates = []

        for sentence_part in sentence_parts:
            interval = match_interval(sentence_part)

            if interval:
                interval['start_date'] = 'OPEN' if interval['start_date'] == 'OPEN' else match_rules(self.now, interval[
                    'start_date'], self.expect_future_day)
                interval['end_date'] = 'OPEN' if interval['end_date'] == 'OPEN' else match_rules(self.now, interval[
                    'end_date'], self.expect_future_day)
                parsed_dates.append(interval)
            else:
                parsed_dates += self._get_implicit_intervall(sentence_part)

        parsed_dates = [extend_start_end(intv) for intv in parsed_dates]

        parsed_dates = [{'start_date': self.assamble_datetime(self.now, parsed_date['start_date'], bottom=True),
                         'end_date': self.assamble_datetime(self.now, parsed_date['end_date'], bottom=False)}
                        for parsed_date in parsed_dates]

        parsed_dates = [intv for intv in parsed_dates if intv['start_date'] or intv['end_date']]

        return parsed_dates



from datetime import time

from hun_date_parser.utils import num_to_word

hours_word = {
    0: 'nulla',
    1: 'hajnali egy',
    2: 'hajnali kettő',
    3: 'hajnali három',
    4: 'hajnali négy',
    5: 'hajnali öt',
    6: 'reggel hat',
    7: 'reggel hét',
    8: 'reggel nyolc',
    9: 'reggel kilenc',
    10: 'délelőtt tíz',
    11: 'délelőtt tizenegy',
    12: 'tizenkét',
    13: 'délután egy',
    14: 'délután kettő',
    15: 'délután három',
    16: 'délután négy',
    17: 'délután öt',
    18: 'este hat',
    19: 'este hét',
    20: 'este nyolc',
    21: 'este kilenc',
    22: 'este tíz',
    23: 'este tizenegy',
}

dayparts = {
    0: 'éjjel',
    3: 'hajnal',
    6: 'reggel',
    10: 'délelőtt',
    12: 'délután',
    18: 'este',
    22: 'éjjel',
    24: 'éjjel'
}


def get_daypart(h: int):
    if h == 12:
        return ''
    for i, (k, v) in enumerate(dayparts.items()):
        if k <= h <= list(dayparts)[i+1]:
            return v


def time2relitivetexttime(t: time, resolution: int):
    assert 1 <= resolution <= 3

    if resolution >= 1:
        hour = t.hour
        hour_rep = hours_word[hour]

    if resolution >= 2:
        minute = t.minute
        minute_res = num_to_word(minute)

    if resolution >= 3:
        second = t.second
        second_res = num_to_word(second)

    if resolution == 1:
        if hour == 0:
            return 'éjfél'
        elif hour == 12:
            return 'dél'
        else:
            return f"{hour_rep} óra"
    elif resolution == 2:
        return f"{hour_rep} óra {minute_res} perc"
    elif resolution == 3:
        return f"{hour_rep} óra {minute_res} perc {second_res} másodperc"


def time2absolutetexttime(t: time, resolution: int):
    assert 1 <= resolution <= 3

    if resolution >= 1:
        hour = t.hour
        hour_rep = num_to_word(hour)

    if resolution >= 2:
        minute = t.minute
        minute_res = num_to_word(minute)

    if resolution >= 3:
        second = t.second
        second_res = num_to_word(second)

    if resolution == 1:
        return f"{hour_rep} óra"
    elif resolution == 2:
        return f"{hour_rep} óra {minute_res} perc"
    elif resolution == 3:
        return f"{hour_rep} óra {minute_res} perc {second_res} másodperc"


def time2digi(t: time, resolution: int):
    assert 1 <= resolution <= 3

    if resolution == 1:
        return f'{t.hour}'
    if resolution == 2:
        return f'{str(t.hour).zfill(2)}:{str(t.minute).zfill(2)}'
    if resolution == 3:
        return f'{str(t.hour).zfill(2)}:{str(t.minute).zfill(2)}:{str(t.second).zfill(2)}'


def time2lifelike(t: time, resolution: int = 2):

    if resolution == 1:
        t = time(t.hour, 0)

    def get_h_rep(h):
        if h % 12 != 0:
            return h % 12
        else:
            return 12

    h, m = t.hour, t.minute

    h_ = h + 1 if h <= 23 else 0

    if m == 0:
        return f'{get_daypart(h)} {get_h_rep(h)} óra'.lstrip()
    elif 0 < m < 10:
        return f'{get_daypart(h)} {get_h_rep(h)} óra után {m} perccel'.lstrip()
    elif 10 <= m < 15:
        return f'{get_daypart(h_)} negyed {get_h_rep(h_)} előtt {15 - m} perccel'.lstrip()
    elif m == 15:
        return f'{get_daypart(h_)} negyed {get_h_rep(h_)}'.lstrip()
    elif 15 < m <= 20:
        return f'{get_daypart(h_)} negyed {get_h_rep(h_)} után {m - 15} perccel'.lstrip()
    elif 20 < m < 30:
        return f'{get_daypart(h_)} fél {get_h_rep(h_)} előtt {30 - m} perccel'.lstrip()
    elif m == 30:
        return f'{get_daypart(h_)} fél {get_h_rep(h_)}'.lstrip()
    elif 30 < m <= 40:
        return f'{get_daypart(h_)} fél {get_h_rep(h_)} után {m - 30} perccel'.lstrip()
    elif 40 < m < 45:
        return f'{get_daypart(h_)} háromnegyed {get_h_rep(h_)} előtt {45 - m} perccel'.lstrip()
    elif m == 45:
        return f'{get_daypart(h_)} háromnegyed {get_h_rep(h_)}'.lstrip()
    elif 45 < m <= 50:
        return f'{get_daypart(h_)} háromnegyed {get_h_rep(h_)} után {m - 45} perccel'.lstrip()
    elif 50 < m:
        return f'{get_daypart(h_)} {get_h_rep(h_)} óra előtt {60 - m} perccel'.lstrip()
    else:
        return ''



import re

from typing import Dict, List, Any, Tuple, Optional
from datetime import datetime

from hun_date_parser.date_parser.patterns import R_DIGI, R_HOUR_MIN, R_HOUR_MIN_REV, R_HWORDS_
from hun_date_parser.utils import remove_accent, word_to_num, Year, Month, Day, Hour, Minute, Daypart
from hun_date_parser.date_parser.date_parsers import match_weekday

NAN = -1


def match_digi_clock(s: str) -> List[Dict[str, Any]]:
    """
    Match digi clock format.
    :param s: textual input
    :return: tuple of date parts
    """
    match = re.findall(R_DIGI, s)

    res = []
    for group in match:
        group = [int(m.lstrip('0')) for m in group if m.lstrip('0')]

        if len(group) == 2:
            h, m = group
            res.append({'match': group, 'date_parts': [Hour(h, 'digi_clock'), Minute(m, 'digi_clock')]})
        elif len(group) == 1:
            res.append({'match': group, 'date_parts': [Hour(group[0], 'digi_clock')]})

    return res


def match_hwords(s: str) -> List[Dict[str, Any]]:
    group = re.findall(R_HWORDS_, s)
    group = [m for m in group if ''.join(m)]

    res = []
    for match in group:
        match = int(match)
        res.append({'match': match, 'date_parts': [Hour(match, 'hwords')]})

    return res


def _raw_match_time_words(s: str) -> Optional[Tuple[Any, Any, Any, Any, Any]]:
    """
    Extracts date and time particles from text
    :param s: input text
    :return:
    """
    group = re.findall(R_HOUR_MIN, s)
    group = [m for m in group if ''.join(m)]

    group_rev = re.findall(R_HOUR_MIN_REV, s)
    group_rev = [m for m in group_rev if ''.join(m)]

    if not (group or group_rev):
        return None
    elif group and not group_rev:
        daypart, hour_modifier, hour, minute = group[0]
    elif not group and group_rev:
        minute, daypart, hour_modifier, hour, is_before = group_rev[0]
        minute += (' ' + is_before)
    elif group_rev[0].count('') < group[0].count(''):
        minute, daypart, hour_modifier, hour, is_before = group_rev[0]
        minute += (' ' + is_before)
    else:
        daypart, hour_modifier, hour, minute = group[0]

    return group, daypart, hour_modifier, hour, minute


def match_time_words(s: str) -> List[Dict[str, Any]]:
    """
    :param s: textual input
    :return: tuple of date parts
    """
    parts = _raw_match_time_words(s)
    if not parts:
        return []
    else:
        group, daypart, hour_modifier, hour, minute = parts

    # Only numbers can match dates as well, this is an attempt to remove false matches
    hour_index = s.index(f'{hour}')
    before_hour = s[:hour_index].split()
    if before_hour:
        months = ['jan', 'feb', 'mar', 'apr', 'maj', 'jun', 'jul', 'aug', 'szep', 'okt', 'nov', 'dec']
        for month in months:
            if month in remove_accent(before_hour[-1]):
                return []

    # Fix false time match for input 'jövő hét'
    if remove_accent(hour) == 'het':
        hour_indeces = [m.start() for m in re.finditer('het(?!fo)', remove_accent(s))]
        if hour_indeces:
            before_hour = s[:hour_indeces[-1]].split()
            if before_hour:
                if 'jovo' in remove_accent(before_hour[-1]):
                    return []

    res = []
    am = True
    date_parts = []

    if daypart and hour:
        if 'reggel' in daypart or 'delelott' in remove_accent(daypart) or 'hajnal' in daypart:
            am = True
        elif 'delutan' in remove_accent(daypart) or 'este' in daypart or 'ejjel' in remove_accent(daypart):
            am = False

    if hour:

        # SKIP the whole matching rule when any of these apply
        # TODO: come up with a more elegant solution for this
        # TODO: i.e: by implementing the possibility of one rule exclude another
        # this is made redundant by the change in the patterns
        non_hours = ['ev', 'perc']
        for nh in non_hours:
            if f' {nh}' in remove_accent(hour) or remove_accent(hour).startswith(nh):
                return []

        if 'mulva' in remove_accent(s):
            return []

        hour_num = word_to_num(hour)
        minute_num = word_to_num(minute)

        if not daypart:
            # default to business hour if daypart is not specified
            if hour_num < 8:
                hour_num += 12

        if hour_modifier:
            if 'haromnegyed' in remove_accent(hour_modifier):
                hour_num = hour_num - 1 if hour_num - 1 >= 0 else 23
                minute_num = 45
            elif 'fel' in remove_accent(hour_modifier):
                hour_num = hour_num - 1 if hour_num - 1 >= 0 else 23
                minute_num = 30
            elif 'negyed' in remove_accent(hour_modifier):
                hour_num = hour_num - 1 if hour_num - 1 >= 0 else 23
                minute_num = 15

        if hour_num == NAN:
            return []
        else:
            if hour_num < 12 and not am:
                hour_num += 12

        if minute or hour_modifier:
            # this is made redundant by the change in the patterns
            non_minutes = ['ev', 'ora']
            for nm in non_minutes:
                if f' {nm}' in remove_accent(minute) or remove_accent(minute).startswith(nm):
                    return []

            if 'elott' in remove_accent(minute) and not hour_modifier:
                hour_num -= (minute_num // 60) + 1
                hour_num = hour_num if hour_num >= 0 else 23
                date_parts.extend([Hour(hour_num, 'time_words'), Minute(60 - (minute_num % 60), 'time_words')])
            elif 'elott' in remove_accent(minute) and hour_modifier:
                n_minutes_before = word_to_num(minute)
                if n_minutes_before != NAN:
                    minute_num -= n_minutes_before
                if minute_num < 0:
                    hour_num += (minute_num // 60)
                    hour_num = hour_num if hour_num >= 0 else 23
                    minute_num = minute_num % 60
                date_parts.extend([Hour(hour_num, 'time_words'), Minute(minute_num, 'time_words')])
            elif hour_modifier:
                n_minutes_after = word_to_num(minute)
                if n_minutes_after != NAN:
                    minute_num += n_minutes_after
                if minute_num > 59:
                    hour_num += (minute_num // 60)
                    hour_num = hour_num if hour_num <= 23 else 0
                    minute_num = minute_num % 60
                date_parts.extend([Hour(hour_num, 'time_words'), Minute(minute_num, 'time_words')])
            else:
                date_parts.extend([Hour(hour_num, 'time_words'), Minute(minute_num, 'time_words')])

        else:
            date_parts.append(Hour(hour_num, 'time_words'))

        res.append({'match': group, 'date_parts': date_parts})

    elif daypart:
        if 'hajnal' in daypart:
            res.append({'match': group, 'date_parts': [Daypart(0, 'time_words')]})
        elif 'reggel' in daypart:
            res.append({'match': group, 'date_parts': [Daypart(1, 'time_words')]})
        elif 'delelott' in remove_accent(daypart):
            res.append({'match': group, 'date_parts': [Daypart(2, 'time_words')]})
        elif 'delutan' in remove_accent(daypart):
            res.append({'match': group, 'date_parts': [Daypart(3, 'time_words')]})
        elif 'este' in daypart:
            res.append({'match': group, 'date_parts': [Daypart(4, 'time_words')]})
        elif 'ejjel' in remove_accent(daypart):
            res.append({'match': group, 'date_parts': [Daypart(5, 'time_words')]})

    return res


def match_now(s: str, now: datetime) -> List[Dict[str, Any]]:
    if match_weekday(s, now):
        return []

    match = re.match(r'.*\bmost\b.*', s.lower())
    if match:
        date_parts = [Year(now.year, 'now'), Month(now.month, 'now'), Day(now.day, 'now'), Hour(now.hour, 'now'),
                      Minute(now.minute, 'now')]
        return [{'match': 'most', 'date_parts': date_parts}]

    return []



import pytest
from datetime import datetime

from hun_date_parser.date_parser.interval_restriction import extract_datetime_within_interval, ExtractWithinRangeSuccess


scenarios = [
        ({'start_date': datetime(2021, 10, 11), 'end_date': datetime(2021, 10, 18, 23, 59, 59)},
         'kedden',
         (ExtractWithinRangeSuccess.VALID_IN_RANGE,
          [{'start_date': datetime(2021, 10, 12), 'end_date': datetime(2021, 10, 12, 23, 59, 59)}])),

        ({'start_date': datetime(2021, 10, 11), 'end_date': datetime(2021, 10, 18, 23, 59, 59)},
         'kedden vagy szerdán',
         (ExtractWithinRangeSuccess.VALID_IN_RANGE,
          [{'start_date': datetime(2021, 10, 12), 'end_date': datetime(2021, 10, 12, 23, 59, 59)}])),

        ({'start_date': datetime(2022, 1, 1), 'end_date': datetime(2022, 12, 31, 23, 59, 59)},
         'februárban',
         (ExtractWithinRangeSuccess.VALID_IN_RANGE,
          [{'start_date': datetime(2022, 2, 1), 'end_date': datetime(2022, 2, 28, 23, 59, 59)}])),

        ({'start_date': datetime(2020, 1, 1), 'end_date': datetime(2020, 12, 31, 23, 59, 59)},
         'februárban',
         (ExtractWithinRangeSuccess.VALID_IN_RANGE,
          [{'start_date': datetime(2020, 2, 1), 'end_date': datetime(2020, 2, 29, 23, 59, 59)}])),

        # Test AM/PM mismatch resolution
        ({'start_date': datetime(2021, 1, 1, 12), 'end_date': datetime(2021, 1, 1, 16, 59, 59)},
         'három órakor',
         (ExtractWithinRangeSuccess.VALID_IN_RANGE,
          [{'start_date': datetime(2021, 1, 1, 15), 'end_date': datetime(2021, 1, 1, 15, 59, 59)}])),

        # Test AM/PM mismatch resolution
        ({'start_date': datetime(2021, 1, 1, 18), 'end_date': datetime(2021, 1, 1, 21, 59, 59)},
         'nyolc órakor',
         (ExtractWithinRangeSuccess.VALID_IN_RANGE,
          [{'start_date': datetime(2021, 1, 1, 20), 'end_date': datetime(2021, 1, 1, 20, 59, 59)}])),

        # Test AM/PM mismatch resolution
        ({'start_date': datetime(2021, 1, 1, 18), 'end_date': datetime(2021, 1, 1, 21, 59, 59)},
         'reggel nyolckor',
         (ExtractWithinRangeSuccess.OUT_OF_RANGE_FALLBACK,
          [{'start_date': datetime(2021, 10, 11, 8), 'end_date': datetime(2021, 10, 11, 8, 59, 59)}])),

        # Test AM/PM mismatch resolution
        ({'start_date': datetime(2021, 1, 1, 20), 'end_date': datetime(2021, 1, 2, 2, 59, 59)},
         '12-kor',
         (ExtractWithinRangeSuccess.VALID_IN_RANGE,
          [{'start_date': datetime(2021, 1, 2, 0), 'end_date': datetime(2021, 1, 2, 0, 59, 59)}])),

        ({'start_date': datetime(2020, 1, 1), 'end_date': datetime(2020, 12, 31, 23, 59, 59)},
         'jövő februárban',
         (ExtractWithinRangeSuccess.OUT_OF_RANGE_FALLBACK,
          [{'start_date': datetime(2022, 2, 1), 'end_date': datetime(2022, 2, 28, 23, 59, 59)}])),

        ({'start_date': datetime(2021, 1, 1), 'end_date': datetime(2021, 3, 31, 23, 59, 59)},
         'augusztusban',
         (ExtractWithinRangeSuccess.OUT_OF_RANGE_FALLBACK,
          [{'start_date': datetime(2021, 8, 1), 'end_date': datetime(2021, 8, 31, 23, 59, 59)}])),

        ({'start_date': datetime(2021, 1, 1), 'end_date': datetime(2021, 3, 31, 23, 59, 59)},
         'augusztustól',
         (ExtractWithinRangeSuccess.OPEN_RANGE_FALLBACK,
          [{'start_date': datetime(2021, 8, 1), 'end_date': None}])),

        ({'start_date': datetime(2021, 1, 1), 'end_date': datetime(2021, 3, 31, 23, 59, 59)},
         'jövő kedd',
         (ExtractWithinRangeSuccess.RELATIVE_TIME_WORD_FALLBACK,
          [{'start_date': datetime(2021, 10, 19), 'end_date': datetime(2021, 10, 19, 23, 59, 59)}])),
    ]


@pytest.mark.parametrize("interval_restriction, query_sentence, expected", scenarios)
def test_extract_within_interval(interval_restriction, query_sentence, expected):
    result = extract_datetime_within_interval(interval_restriction['start_date'],
                                              interval_restriction['end_date'],
                                              query_sentence,
                                              fallback_now=datetime(2021, 10, 11))

    assert result == expected



"""
This module (still WIP) implements extracting datetime intervals from pre-determined datetime intervals.
If extraction within the set interval is not successful, the extraction falls back to the text2datetime function
"""

from datetime import datetime, date, time, timedelta
from enum import Enum
from typing import Dict, List, Union, Tuple
from copy import deepcopy

from hun_date_parser import text2datetime
from hun_date_parser.utils import remove_accent
from hun_date_parser.date_parser.time_parsers import _raw_match_time_words

datelike = Union[datetime, date, time, None]


def get_reversed_am_pm(dt: datetime):
    dt_ = deepcopy(dt)
    if 0 < dt.hour < 12:
        dt_ = datetime(dt.year, dt.month, dt.day, dt.hour + 12, dt.minute, dt.second)
    elif not (dt.hour == 23 and dt.minute == 59) and dt.hour > 12:
        dt_ = datetime(dt.year, dt.month, dt.day, dt.hour - 12, dt.minute, dt.second)
    elif dt.hour == 12:
        dt_ = dt_ + timedelta(days=1)
        dt_ = datetime(dt_.year, dt_.month, dt_.day, 0, dt_.minute, dt_.second)

    return dt_


class ExtractWithinRangeSuccess(Enum):
    VALID_IN_RANGE = 'in_range'
    OUT_OF_RANGE_FALLBACK = 'out_of_range'
    RELATIVE_TIME_WORD_FALLBACK = 'relative_time_word'
    OPEN_RANGE_FALLBACK = 'open_range'
    NO_MATCH_FALLBACK = 'no_match'


def is_relative_datetime(query: str):
    """
    Determines wheter the query text refers to a relative datetime, ie.: next monday
    """
    relative_time_words = ['ma', 'holnap', 'holnaput', 'tegnap', 'tegnapel', 'jovo', 'mult']
    transformed_query = remove_accent(query)

    for time_word in relative_time_words:
        if time_word in transformed_query:
            return True

    return False


def extract_datetime_within_interval(interval_start: datetime,
                                     interval_end: datetime,
                                     query_text: str,
                                     expect_future_day: bool = False,
                                     fallback_now=datetime.now()) -> Tuple[ExtractWithinRangeSuccess,
                                                                           List[Dict[str, datelike]]]:
    """
    Extracts datetime intervals from pre-determined datetime intervals.
    :param interval_start: Bounding interval start
    :param interval_end: Bounding interval end
    :param query_text: Text to extract datetime interval from
    :param expect_future_day: Shows if the extracted date will be offset.
    :param fallback_now: When interval restriction is unsuccessful (RELATIVE_TIME_WORD_FALLBACK, OUT_OF_RANGE_FALLBACK)
    the text2datetime function's result is returned, which can be supplied with a pseudo-now datetime
    :return: success flag, restricted time interval
    """

    res = text2datetime(query_text,
                        expect_future_day=expect_future_day,
                        now=interval_start)

    possible_am_pm_missmatch = False
    parts = _raw_match_time_words(query_text)
    if parts:
        group, daypart, hour_modifier, hour, minute = parts
        if not daypart and hour:
            possible_am_pm_missmatch = True

    extend_res = []
    if possible_am_pm_missmatch:
        for match in res:
            if isinstance(match["start_date"], datetime) and isinstance(match["end_date"], datetime):
                extend_res.append({
                        "start_date": get_reversed_am_pm(match["start_date"]),
                        "end_date": get_reversed_am_pm(match["end_date"])
                    })

    res += extend_res

    restricted_date: List[Dict[str, datelike]] = []
    response_candidates = [(5, ExtractWithinRangeSuccess.NO_MATCH_FALLBACK, restricted_date)]
    for r in res:
        if not (isinstance(r['start_date'], datetime) and isinstance(r['end_date'], datetime)):
            response_type = ExtractWithinRangeSuccess.OPEN_RANGE_FALLBACK
            restricted_date = text2datetime(query_text,
                                            expect_future_day=expect_future_day,
                                            now=fallback_now)

            response_candidates.append((4, response_type, restricted_date))
            continue

        assert type(interval_start) == datetime and type(r['start_date']) == datetime
        assert type(interval_end) == datetime and type(r['end_date']) == datetime
        if not (interval_start <= r['start_date'] and r['end_date'] <= interval_end):
            # Extracted datetime is out of expected interval...
            response_type = ExtractWithinRangeSuccess.OUT_OF_RANGE_FALLBACK
            restricted_date = text2datetime(query_text,
                                            expect_future_day=expect_future_day,
                                            now=fallback_now)

            response_candidates.append((2, response_type, restricted_date))
            continue

        if is_relative_datetime(query_text):
            # Datetime ranges relative to the current timestamp doesn't really make sense in this scenario...
            response_type = ExtractWithinRangeSuccess.RELATIVE_TIME_WORD_FALLBACK
            restricted_date = text2datetime(query_text,
                                            expect_future_day=expect_future_day,
                                            now=fallback_now)

            response_candidates.append((3, response_type, restricted_date))
        else:
            response_type = ExtractWithinRangeSuccess.VALID_IN_RANGE
            restricted_date = [r]

            response_candidates.append((1, response_type, restricted_date))

    response_candidates_ranked = sorted(response_candidates, key=lambda x: x[0])
    _, response_type, restricted_date = response_candidates_ranked[0]

    return response_type, restricted_date



import re

from typing import Dict, List

from .patterns import R_MULTI, R_TOLIG, R_TOL, R_IG


def match_multi_match(s: str) -> List[str]:
    match = re.match(R_MULTI, s)

    if match:
        groups = match.groups()
        groups = [m.rstrip().lstrip() for m in groups if m]

        return groups

    return [s]


def match_interval(s: str) -> Dict:
    match = re.match(R_TOLIG, s)
    if match:
        groups = match.groups()
        groups = [m.lstrip().rstrip() for m in groups if m]

        if len(groups) == 2:
            return {
                'start_date': groups[0],
                'end_date': groups[1]
            }

    match = re.match(R_TOL, s)
    if match:
        groups = match.groups()
        groups = [m.lstrip().rstrip() for m in groups if m]

        if len(groups) == 1:
            return {
                'start_date': groups[0],
                'end_date': 'OPEN'
            }

    match = re.match(R_IG, s)
    if match:
        groups = match.groups()
        groups = [m.lstrip().rstrip() for m in groups if m]

        if len(groups) == 1:
            return {
                'start_date': 'OPEN',
                'end_date': groups[0]
            }

    return {}



import re
from typing import Dict, List, Any
from datetime import datetime, timedelta

from .patterns import (R_ISO_DATE, R_NAMED_MONTH, R_TODAY, R_TOMORROW, R_NTOMORROW, R_YESTERDAY, R_NYESTERDAY,
                       R_WEEKDAY, R_WEEK, R_YEAR, R_NDAYS_FROM_NOW, R_NWEEKS_FROM_NOW, R_NHOURS_FROM_NOW,
                       R_NMINS_FROM_NOW)
from hun_date_parser.utils import remove_accent, word_to_num, Year, Month, Week, Day, Hour, Minute


def match_iso_date(s: str) -> List[Dict[str, Any]]:
    """
    Match ISO date-like format.
    :param s: textual input
    :return: tuple of date parts
    """
    match = re.findall(R_ISO_DATE, s)

    res = []
    if match:
        for group in match:
            group = [int(m.lstrip('0')) for m in group if m.lstrip('0')]

            if len(group) == 1:
                res.append({'match': group, 'date_parts': [Year(group[0], 'match_iso_date')]})
            elif len(group) == 2:
                res.append({'match': group,
                            'date_parts': [Year(group[0], 'match_iso_date'), Month(group[1], 'match_iso_date')]})
            elif len(group) == 3:
                res.append({'match': group,
                            'date_parts': [Year(group[0], 'match_iso_date'), Month(group[1], 'match_iso_date'),
                                           Day(group[2], 'match_iso_date')]})

    return res


def match_named_month(s: str, now: datetime) -> List[Dict[str, Any]]:
    """
    Match named month and day
    :param s: textual input
    :return: tuple of date parts
    """
    groups = re.findall(R_NAMED_MONTH, s)
    months = ['jan', 'feb', 'mar', 'apr', 'maj', 'jun', 'jul', 'aug', 'szep', 'okt', 'nov', 'dec']

    res = []
    groups = [(mod, m, d.lstrip('0')) if d else (mod, m, '') for mod, m, d in groups]
    for group in groups:
        group_res = {'match': group, 'date_parts': []}
        if group[0]:
            if 'jovo' in remove_accent(group[0]):
                group_res['date_parts'].append(Year(now.year + 1, 'named_month'))
            elif 'tavaly' in remove_accent(group[0]):
                group_res['date_parts'].append(Year(now.year - 1, 'named_month'))

        for i, month in enumerate(months):
            if month in remove_accent(group[1]):
                group_res['date_parts'].append(Month(i + 1, 'named_month'))
                break

        if group[2]:
            group_res['date_parts'].append(Day(int(group[2]), 'named_month'))

        res.append(group_res)

    return res


def match_relative_day(s: str, now: datetime) -> List[Dict[str, Any]]:
    groups = [*re.findall(R_TODAY, s),
              *re.findall(R_TOMORROW, s),
              *re.findall(R_NTOMORROW, s),
              *re.findall(R_YESTERDAY, s),
              *re.findall(R_NYESTERDAY, s)]

    res = []
    for group in groups:

        if type(group) != str:
            group = [m for m in group if m][0]

        if 'ma' in group or 'má' in group:
            res.append({'match': group, 'date_parts': [Year(now.year, 'relative_day'), Month(now.month, 'relative_day'),
                                                       Day(now.day, 'relative_day')]})
        elif 'holnapu' in group:
            tom2 = now + timedelta(days=2)
            res.append({'match': group,
                        'date_parts': [Year(tom2.year, 'relative_day'), Month(tom2.month, 'relative_day'),
                                       Day(tom2.day, 'relative_day')]})
        elif 'holnap' in group:
            tom = now + timedelta(days=1)
            res.append({'match': group, 'date_parts': [Year(tom.year, 'relative_day'), Month(tom.month, 'relative_day'),
                                                       Day(tom.day, 'relative_day')]})
        elif 'tegnapel' in group:
            yes2 = now - timedelta(days=2)
            res.append({'match': group,
                        'date_parts': [Year(yes2.year, 'relative_day'), Month(yes2.month, 'relative_day'),
                                       Day(yes2.day, 'relative_day')]})
        elif 'tegnap' in group:
            yes = now - timedelta(days=1)
            res.append({'match': group, 'date_parts': [Year(yes.year, 'relative_day'), Month(yes.month, 'relative_day'),
                                                       Day(yes.day, 'relative_day')]})

    return res


def match_weekday(s: str, now: datetime, expect_future_day: bool = False) -> List[Dict[str, Any]]:
    groups = re.findall(R_WEEKDAY, s)

    res = []
    for group in groups:
        date_parts = {'match': group, 'date_parts': []}
        week, day = group
        n_weeks = 0

        if 'jovo' in remove_accent(week):
            n_weeks = 1
        elif 'mult' in remove_accent(week) or 'elozo' in remove_accent(week):
            n_weeks = -1

        def offset_date(date):
            if date.strftime("%Y-%m-%d") < now.strftime("%Y-%m-%d"):
                return date + timedelta(days=7)
            return date

        def get_day_of_week(w, d):
            return ((now - timedelta(days=now.weekday())) + timedelta(days=w * 7)) + timedelta(days=d)

        if 'hetfo' in remove_accent(day):
            day = offset_date(get_day_of_week(n_weeks, 0)) if expect_future_day and n_weeks >= 0\
                else get_day_of_week(n_weeks, 0)
            date_parts['date_parts'] = [Year(day.year, 'weekday'), Month(day.month, 'weekday'), Day(day.day, 'weekday')]
        elif 'kedd' in remove_accent(day):
            day = offset_date(get_day_of_week(n_weeks, 1)) if expect_future_day and n_weeks >= 0 \
                else get_day_of_week(n_weeks, 1)
            date_parts['date_parts'] = [Year(day.year, 'weekday'), Month(day.month, 'weekday'), Day(day.day, 'weekday')]
        elif 'szerda' in remove_accent(day):
            day = offset_date(get_day_of_week(n_weeks, 2)) if expect_future_day and n_weeks >= 0 \
                else get_day_of_week(n_weeks, 2)
            date_parts['date_parts'] = [Year(day.year, 'weekday'), Month(day.month, 'weekday'), Day(day.day, 'weekday')]
        elif 'csut' in remove_accent(day):
            day = offset_date(get_day_of_week(n_weeks, 3)) if expect_future_day and n_weeks >= 0 \
                else get_day_of_week(n_weeks, 3)
            date_parts['date_parts'] = [Year(day.year, 'weekday'), Month(day.month, 'weekday'), Day(day.day, 'weekday')]
        elif 'pent' in remove_accent(day):
            day = offset_date(get_day_of_week(n_weeks, 4)) if expect_future_day and n_weeks >= 0 \
                else get_day_of_week(n_weeks, 4)
            date_parts['date_parts'] = [Year(day.year, 'weekday'), Month(day.month, 'weekday'), Day(day.day, 'weekday')]
        elif 'szom' in remove_accent(day):
            day = offset_date(get_day_of_week(n_weeks, 5)) if expect_future_day and n_weeks >= 0 \
                else get_day_of_week(n_weeks, 5)
            date_parts['date_parts'] = [Year(day.year, 'weekday'), Month(day.month, 'weekday'), Day(day.day, 'weekday')]
        elif 'vas' in remove_accent(day):
            day = offset_date(get_day_of_week(n_weeks, 6)) if expect_future_day and n_weeks >= 0 \
                else get_day_of_week(n_weeks, 6)
            date_parts['date_parts'] = [Year(day.year, 'weekday'), Month(day.month, 'weekday'), Day(day.day, 'weekday')]

        res.append(date_parts)

    return res


def match_week(s: str, now: datetime) -> List[Dict[str, Any]]:
    groups = re.findall(R_WEEK, s)

    res = []
    for group in groups:
        date_parts = {'match': group, 'date_parts': []}

        if 'ez' in group:
            y, w = now.isocalendar()[0:2]
            date_parts['date_parts'].extend([Year(y, 'week'), Week(w, 'week')])
        elif 'jovo' in remove_accent(group):
            y, w = (now + timedelta(days=7)).isocalendar()[0:2]
            date_parts['date_parts'].extend([Year(y, 'week'), Week(w, 'week')])
        elif 'mult' in remove_accent(group) or 'elozo' in remove_accent(group):
            y, w = (now - timedelta(days=7)).isocalendar()[0:2]
            date_parts['date_parts'].extend([Year(y, 'week'), Week(w, 'week')])

        res.append(date_parts)

    return res


def match_n_periods_compared_to_now(s: str, now: datetime) -> List[Dict[str, Any]]:
    # TODO: implement n-periods-before-now functionality
    fn = 'n_date_periods_compared_to_now'

    regexes = [
        (R_NWEEKS_FROM_NOW, 'w'),
        (R_NDAYS_FROM_NOW, 'd'),
        (R_NHOURS_FROM_NOW, 'h'),
        (R_NMINS_FROM_NOW, 'm')
    ]
    res = []

    for regex, freq in regexes:
        groups = re.findall(regex, s)
        for group in groups:
            date_parts = {'match': group, 'date_parts': []}

            n = group[1]
            if n:
                n = word_to_num(n)
                if freq == 'w':
                    res_dt = (now + timedelta(days=7*n))
                    y, m, d = res_dt.year, res_dt.month, res_dt.day
                    date_parts['date_parts'].extend([Year(y, fn), Month(m, fn), Day(d, fn)])
                elif freq == 'd':
                    res_dt = (now + timedelta(days=n))
                    y, m, d = res_dt.year, res_dt.month, res_dt.day
                    date_parts['date_parts'].extend([Year(y, fn), Month(m, fn), Day(d, fn)])
                elif freq == 'h':
                    res_dt = (now + timedelta(hours=n))
                    y, m, d, h = res_dt.year, res_dt.month, res_dt.day, res_dt.hour
                    date_parts['date_parts'].extend([Year(y, fn), Month(m, fn), Day(d, fn), Hour(h, fn)])
                elif freq == 'm':
                    res_dt = (now + timedelta(minutes=n))
                    y, mo, d, h, mi = res_dt.year, res_dt.month, res_dt.day, res_dt.hour, res_dt.minute
                    date_parts['date_parts'].extend([Year(y, fn), Month(mo, fn),
                                                     Day(d, fn), Hour(h, fn), Minute(mi, fn)])

            res.append(date_parts)

    return res


def match_named_year(s: str, now: datetime) -> List[Dict[str, Any]]:
    groups = re.findall(R_YEAR, s)

    res = []
    for group in groups:
        date_parts = {'match': group, 'date_parts': []}

        if 'tavalyelott' in remove_accent(group):
            date_parts['date_parts'] = [Year(now.year - 2, 'named_year')]
        elif 'tavaly' in remove_accent(group):
            date_parts['date_parts'] = [Year(now.year - 1, 'named_year')]
        elif 'iden' in remove_accent(group):
            date_parts['date_parts'] = [Year(now.year, 'named_year')]
        elif 'jovo' in remove_accent(group):
            date_parts['date_parts'] = [Year(now.year + 1, 'named_year')]
        elif 'mulva' in remove_accent(group):
            num_after = word_to_num(group)
            if num_after != -1:
                date_parts['date_parts'] = [Year(now.year + num_after, 'named_year')]
        elif 'ezelott' in remove_accent(group) or 'korabban' in remove_accent(group):
            num_before = word_to_num(group)
            if num_before != -1:
                date_parts['date_parts'] = [Year(now.year - num_before, 'named_year')]

        res.append(date_parts)

    return res



from datetime import datetime, date, time

from hun_date_parser import datetime2text, text2datetime, text2date, text2time


def test_datetime2text():
    candidates = datetime2text(datetime(2020, 12, 21))

    assert set(candidates) == {'dates', 'times'}
    assert len([c for c in candidates['dates'] if c]) == 2
    assert len([c for c in candidates['times'] if c]) == 4


def test_text2datetime():
    now = datetime(2020, 12, 27)
    tf = [('ma', [{'start_date': datetime(2020, 12, 27), 'end_date': datetime(2020, 12, 27, 23, 59, 59)}]),
          ('ma reggel', [{'start_date': datetime(2020, 12, 27, 6), 'end_date': datetime(2020, 12, 27, 10, 59, 59)}])]

    for inp, out in tf:
        assert text2datetime(inp, now=now) == out


def test_text2date():
    now = datetime(2020, 12, 27)
    tf = [('ma', [{'start_date': date(2020, 12, 27), 'end_date': date(2020, 12, 27)}]),
          ('ma reggel', [{'start_date': date(2020, 12, 27), 'end_date': date(2020, 12, 27)}]),
          ('reggel nyolc óra', [])]

    for inp, out in tf:
        assert text2date(inp, now=now) == out


def test_text2time():
    now = datetime(2020, 12, 27)
    tf = [('ma', []),
          ('ma reggel', [{'start_date': time(6), 'end_date': time(10, 59, 59)}]),
          ('ma délelőtt', [{'start_date': time(8), 'end_date': time(11, 59, 59)}]),
          ('reggel nyolc óra', [{'start_date': time(8), 'end_date': time(8, 59, 59)}])]

    for inp, out in tf:
        assert text2time(inp, now=now) == out



import setuptools
import re
import hun_date_parser

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
    long_description = re.sub(r':.*:', '', long_description)  # remove Github emojis

setuptools.setup(
    name="hun-date-parser",
    version=hun_date_parser.__version__,
    author="Soma Nagy",
    author_email="nagysomabalint@gmail.com",
    description="A tool for extracting datetime intervals from Hungarian sentences and turning datetime objects into Hungarian text.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/szegedai/hun-date-parser",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



from hun_date_parser.date_textualizer.datetime_textualizer import DatetimeTextualizer, datetime2text
from hun_date_parser.date_parser.datetime_extractor import DatetimeExtractor, text2datetime, text2date, text2time

__all__ = ["DatetimeTextualizer", "DatetimeExtractor", "datetime2text", "text2datetime", "text2date", "text2time"]

__version__ = "0.0.11-rc2"



import pytest
from datetime import time, datetime

from hun_date_parser.date_textualizer.time2text import time2lifelike, time2digi, time2relitivetexttime, \
    time2absolutetexttime
from hun_date_parser.date_textualizer.date2text import date2text

tf_time2lifelike = [(time(22, 50), 'éjjel háromnegyed 11 után 5 perccel'),
                    (time(23, 44), 'éjjel háromnegyed 12 előtt 1 perccel'),
                    (time(8, 0), 'reggel 8 óra'),
                    (time(0), 'éjjel 12 óra'),
                    (time(10, 59), 'délelőtt 11 óra előtt 1 perccel'),
                    (time(4, 4), 'hajnal 4 óra után 4 perccel'),
                    (time(2, 16), 'éjjel negyed 3 után 1 perccel'),
                    (time(12), '12 óra'),
                    (time(11, 45), 'háromnegyed 12')]

tf_time2lifelike_res = [(time(22, 50), 2, 'éjjel háromnegyed 11 után 5 perccel'),
                        (time(22, 50), 1, 'este 10 óra'),
                        (time(11, 59, 59), 1, 'délelőtt 11 óra')]

tf_relitivetexttime = [(time(22, 50), 1, 'este tíz óra'),
                       (time(22, 50), 2, 'este tíz óra ötven perc'),
                       (time(22, 50), 3, 'este tíz óra ötven perc nulla másodperc')]

tf_absolutetexttime = [(time(22, 50), 1, 'huszonkettő óra'),
                       (time(22, 50), 2, 'huszonkettő óra ötven perc'),
                       (time(22, 50), 3, 'huszonkettő óra ötven perc nulla másodperc')]

tf_digi = [(time(22, 50), 2, '22:50'),
           (time(22, 50), 3, '22:50:00')]

tf_date = [(datetime(2020, 12, 27), 'ma'),
           (datetime(2020, 12, 28), 'holnap'),
           (datetime(2020, 12, 21), 'ezen a héten hétfőn'),
           (datetime(2020, 12, 20), 'múlt héten vasárnap'),
           (datetime(2020, 12, 19), 'múlt héten szombaton'),
           (datetime(2020, 12, 29), 'jövő hét kedden'),
           (datetime(2020, 12, 13), 'két hete vasárnap')]


@pytest.mark.parametrize("inp,out", tf_time2lifelike)
def test_time2lifelike(inp, out):
    assert time2lifelike(inp) == out


@pytest.mark.parametrize("inp,res,out", tf_time2lifelike_res)
def test_time2lifelike_res(inp, res, out):
    assert time2lifelike(inp, res) == out


@pytest.mark.parametrize("inp,res,out", tf_relitivetexttime)
def test_time2relitivetexttime(inp, res, out):
    assert time2relitivetexttime(inp, resolution=res) == out


@pytest.mark.parametrize("inp,res,out", tf_absolutetexttime)
def test_time2absolutetexttime(inp, res, out):
    assert time2absolutetexttime(inp, resolution=res) == out


@pytest.mark.parametrize("inp,res,out", tf_digi)
def test_time2digi(inp, res, out):
    assert time2digi(inp, resolution=res) == out


@pytest.mark.parametrize("inp,out", tf_date)
def test_date2text(inp, out):
    now = datetime(2020, 12, 27)
    assert date2text(inp, now=now) == out
